//
//  ApiFunctions.swift
//  oin_pos_plugin
//
//  Created by 逸唐陳 on 2020/7/27.
//

import Foundation

class ApiFunctions {
  
  private static var invoiceDate: String {
    var calendar = Calendar.current
    calendar = .current
    let year = calendar.component(.year, from: Date())
    let month = calendar.component(.month, from: Date())
    let newYear = String(year - 1911)
    let strDate = newYear + "年" + invoiceMonthRange(month)
    return strDate
  }
  
  private static func invoiceMonthRange(_ month: Int) -> String {
    switch month {
    case 1, 2:
      return "11-12月"
    case 3, 4:
      return "01-02月"
    case 5, 6:
      return "03-04月"
    case 7, 8:
      return "05-06月"
    case 9, 10:
      return "07-08月"
    case 11, 12:
      return "09-10月"
    default:
      return ""
    }
  }
  
  private static var currentDate: String {
    let date = Date()
    let dateFormatter = DateFormatter()
    dateFormatter.dateFormat = "yyyy/MM/dd HH:mm"
    return dateFormatter.string(from: date)
  }
  
  static func createReceipts(_ emulation: StarIoExtEmulation, receipts: Receipts) -> Data {
    let space: Data = "   ".data(using: .ascii)!
    let space2: Data = "  ".data(using: .ascii)!
    let rect = CGRect(x: 0, y: 0, width: 576,  height: 210)
    let builder: ISCBBuilder = StarIoExt.createCommandBuilder(emulation)
    let encoding = String.Encoding(rawValue: CFStringConvertEncodingToNSStringEncoding(CFStringEncoding(CFStringEncodings.big5.rawValue)))
    let barcode: Data = "\(receipts.invoiceNumber) 20200729".data(using: encoding)!
    builder.beginDocument()
    builder.appendCharacterSpace(0)
    builder.appendAlignment(SCBAlignmentPosition.center)
    builder.appendEmphasis(true)
    builder.appendCharacterSpace(5)
    builder.appendData(withMultipleHeight: "\(receipts.storeName)\n".data(using: encoding), height: 2)
    builder.appendCharacterSpace(0)
    builder.appendEmphasis(false)
    builder.append("---------------------------------\n".data(using: encoding))
    builder.appendData(withMultiple: (
      "電子發票證明聯\n" +
        "\(invoiceDate)\n" +
      "\(receipts.invoiceNumber)\n").data(using: encoding), width: 2, height: 2)
    builder.appendAlignment(SCBAlignmentPosition.left)
    builder.append((
      "\(currentDate)\n" +
        "隨機碼 : 9999       總計 : \(receipts.total)\n" +
        "賣方 : 99999999\n").data(using: encoding))
    builder.appendBarcodeData(withAlignment: barcode, symbology: .code39, width: .extMode1, height: 40, hri: false, position: .center)
    builder.beginPageMode(rect, rotation: SCBBitmapConverterRotation.normal)
    builder.append(space2)
    builder.appendQrCodeData("http://www.star-m.jp/eng/index.html".data(using: String.Encoding.ascii), model: .no2, level: .Q, cell: 5)
    builder.append(space)
    builder.appendQrCodeData("http://www.star-m.jp/eng/index.html".data(using: String.Encoding.ascii), model: .no2, level: .Q, cell: 5)
    builder.endPageMode()
    builder.appendCutPaper(SCBCutPaperAction.partialCutWithFeed)
//     orders
    builder.appendAlignment(.center)
    builder.appendEmphasis(true)
    builder.appendData(withMultipleHeight: "消費明細-內用A4\n".data(using: encoding), height: 2)
    builder.appendEmphasis(false)
    for order in receipts.orders {
      let rect = CGRect(x: 20, y: 0, width: 576,  height: order.comboHeight)
      builder.beginPageMode(rect, rotation: .normal)
      builder.appendData(withAlignment: order.name.data(using: encoding), position: .left)
      builder.appendAbsolutePosition(200)
      builder.append("*\(order.amount)".data(using: encoding))
      builder.appendAbsolutePosition(300)
      builder.appendData(withAlignment: "$\(order.price)\n".data(using: encoding), position: .right)
      if let combo = order.combo {
        for detail in combo {
          builder.append("  \(detail)\n".data(using: encoding))
        }
      }
      builder.endPageMode()
    }
    builder.appendEmphasis(true)
    builder.append("\n\n發票金額 $11,000\n".data(using: encoding))
    builder.appendEmphasis(false)
    builder.append("現金 $12,000   找零 $1,000\n".data(using: encoding))
    builder.append("退貨憑電子發票聯正本辦理\n".data(using: encoding))
    builder.appendCutPaper(SCBCutPaperAction.partialCutWithFeed)
    builder.endDocument()
    return builder.commands.copy() as! Data
  }
  
  static func createPeripheralData(_ emulation: StarIoExtEmulation) -> Data {
    let builder: ISCBBuilder = StarIoExt.createCommandBuilder(emulation)
    
    builder.beginDocument()
    
    builder.appendPeripheral(SCBPeripheralChannel.no1)
    builder.appendPeripheral(SCBPeripheralChannel.no2)
    builder.appendPeripheral(SCBPeripheralChannel.no1, time: 2000)
    builder.appendPeripheral(SCBPeripheralChannel.no2, time: 2000)
    
    builder.endDocument()
    
    return builder.commands.copy() as! Data
  }
  
  static func append3inchTextReceiptData(_ emulation: StarIoExtEmulation) -> Data {
    let space: Data = "   ".data(using: .ascii)!
    let space2: Data = " ".data(using: .ascii)!
    let otherDataNw7: Data = "A0123456789B".data(using: String.Encoding.ascii)!
    let rect = CGRect(x: 0, y: 0, width: 576,  height: 240)
    let builder: ISCBBuilder = StarIoExt.createCommandBuilder(emulation)
    let encoding = String.Encoding(rawValue: CFStringConvertEncodingToNSStringEncoding(CFStringEncoding(CFStringEncodings.big5.rawValue)))
    builder.beginDocument()
    builder.appendCharacterSpace(0)
    builder.appendAlignment(SCBAlignmentPosition.center)
    builder.appendEmphasis(true)
    builder.appendCharacterSpace(5)
    builder.appendData(withMultipleHeight: "碳佐麻里精品燒肉\n".data(using: encoding), height: 2)
    builder.appendCharacterSpace(0)
    builder.appendEmphasis(false)
    builder.append("---------------------------------\n".data(using: encoding))
    builder.appendData(withMultiple: (
      "電子發票證明聯\n" +
        "103年01-02月\n" +
      "EV-99999999\n").data(using: encoding), width: 2, height: 2)
    builder.appendAlignment(SCBAlignmentPosition.left)
    builder.append((
      "2014/01/15 13:00\n" +
        "隨機碼 : 9999       總計 : 999\n" +
        "賣方 : 99999999\n").data(using: encoding))
    builder.appendBarcodeData(withAlignment: otherDataNw7, symbology: .NW7, width: .extMode1, height: 40, hri: false, position: .center)
    builder.beginPageMode(rect, rotation: SCBBitmapConverterRotation.normal)
    builder.append(space)
    builder.appendQrCodeData("http://www.star-m.jp/eng/index.html".data(using: String.Encoding.ascii), model: .no2, level: .Q, cell: 4)
    builder.append(space)
    builder.append(space)
    builder.append(space2)
    builder.appendQrCodeData("http://www.star-m.jp/eng/index.html".data(using: String.Encoding.ascii), model: .no2, level: .Q, cell: 4)
    builder.endPageMode()
    
    //        builder.appendAlignment(SCBAlignmentPosition.left)
    //
    //        builder.append((
    //            "\n" +
    //            "商品退換請持本聯及銷貨明細表。\n" +
    //            "9999999-9999999 999999-999999 9999\n" +
    //            "\n").data(using: encoding))
    //
    //        builder.appendData(withAlignment: "銷貨明細表 　(銷售)\n".data(using: encoding), position: SCBAlignmentPosition.center)
    //
    //        builder.appendData(withAlignment: "2014-01-15 13:00:02\n".data(using: encoding), position: SCBAlignmentPosition.right)
    //
    //        builder.append((
    //            "\n" +
    //            "烏龍袋茶2g20入                55 x2 110TX\n" +
    //            "茉莉烏龍茶2g20入              55 x2 110TX\n" +
    //            "天仁觀音茶2g*20               55 x2 110TX\n").data(using: encoding))
    //
    //        builder.appendData(withEmphasis: (
    //            "      小　 計 :                330\n" +
    //            "      總   計 :                330\n").data(using: encoding))
    //
    //        builder.append((
    //            "------------------------------------------------\n" +
    //            "現 金                          400\n" +
    //            "      找　 零 :                 70\n").data(using: encoding))
    //
    //        builder.appendData(withEmphasis: " 101 發票金額 :                330\n".data(using: encoding))
    //
    //        builder.append((
    //            "2014-01-15 13:00\n" +
    //            "\n").data(using: encoding))
    //
    //        builder.appendAlignment(SCBAlignmentPosition.center)
    //
    ////      builder.appendBarcodeData("{BStar.".data(using: encoding),              symbology: SCBBarcodeSymbology.Code128, width: SCBBarcodeWidth.Mode2, height: 40, hri: true)
    //        builder.appendBarcodeData("{BStar.".data(using: String.Encoding.ascii), symbology: SCBBarcodeSymbology.code128, width: SCBBarcodeWidth.mode2, height: 40, hri: true)
    //
    //        builder.appendAlignment(SCBAlignmentPosition.left)
    //
    //        builder.append((
    //            "\n" +
    //            "商品退換、贈品及停車兌換請持本聯。\n" +
    //            "9999999-9999999 999999-999999 9999\n").data(using: encoding))
    builder.appendCutPaper(SCBCutPaperAction.partialCutWithFeed)
    builder.endDocument()
    
    return builder.commands.copy() as! Data
  }
  
  static func createImageData(_ emulation: StarIoExtEmulation, _ info: UIImage, _ detail: UIImage) -> Data {
    let builder: ISCBBuilder = StarIoExt.createCommandBuilder(emulation)
    
    builder.beginDocument()
    builder.appendBitmap(withAlignment: info, diffusion: true, position: SCBAlignmentPosition.center)
    builder.appendCutPaper(SCBCutPaperAction.partialCutWithFeed)
    builder.appendBitmap(withAlignment: detail, diffusion: true, position: SCBAlignmentPosition.center)
    builder.appendCutPaper(SCBCutPaperAction.partialCutWithFeed)
    builder.endDocument()
    
    return builder.commands.copy() as! Data
  }
  
}
