//
//  ReceiptsDetailCell.swift
//  oin_pos_plugin
//
//  Created by 逸唐陳 on 2020/8/21.
//

import UIKit

class ReceiptsDetailCell: UITableViewCell {
  
  let title: UILabel = {
    let label = UILabel()
    label.translatesAutoresizingMaskIntoConstraints = false
    label.textColor = .black
    label.font = UIFont.systemFont(ofSize: 25, weight: .regular)
    return label
  }()
  
  let separator: UIView = {
    let view = UIView()
    view.translatesAutoresizingMaskIntoConstraints = false
    view.backgroundColor = .black
    return view
  }()
  
  lazy var tableView: UITableView = {
    let view = UITableView(frame: .zero, style: .plain)
    view.translatesAutoresizingMaskIntoConstraints = false
    view.separatorStyle = .none
    view.backgroundColor = .white
    view.tableFooterView = UIView()
    view.delegate = self
    view.dataSource = self
    view.isScrollEnabled = false
    return view
  }()
  
  var orders = [Order]()
  
  override init(style: UITableViewCell.CellStyle, reuseIdentifier: String?) {
    super.init(style: style, reuseIdentifier: reuseIdentifier)
    selectionStyle = .none
    backgroundColor = .white
    contentView.addSubview(title)
    title.leadingAnchor.constraint(equalTo: leadingAnchor, constant: 10).isActive = true
    title.trailingAnchor.constraint(equalTo: trailingAnchor, constant: -10).isActive = true
    title.heightAnchor.constraint(equalToConstant: 30).isActive = true
    title.topAnchor.constraint(equalTo: contentView.topAnchor).isActive = true
    contentView.addSubview(separator)
    separator.leadingAnchor.constraint(equalTo: leadingAnchor).isActive = true
    separator.trailingAnchor.constraint(equalTo: trailingAnchor).isActive = true
    separator.topAnchor.constraint(equalTo: title.bottomAnchor, constant: 20).isActive = true
    separator.heightAnchor.constraint(equalToConstant: 2).isActive = true
    contentView.addSubview(tableView)
    tableView.topAnchor.constraint(equalTo: separator.bottomAnchor).isActive = true
    tableView.leadingAnchor.constraint(equalTo: leadingAnchor).isActive = true
    tableView.trailingAnchor.constraint(equalTo: trailingAnchor).isActive = true
    tableView.bottomAnchor.constraint(equalTo: bottomAnchor).isActive = true
    tableView.register(DetailCell.self, forCellReuseIdentifier: "cell")
  }
  
  required init?(coder: NSCoder) {
    fatalError("init(coder:) has not been implemented")
  }
}

extension ReceiptsDetailCell: UITableViewDelegate {
  func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
    let cell = tableView.cellForRow(at: indexPath) as! DetailCell
    print(cell.combo.frame.height)
  }
}
extension ReceiptsDetailCell: UITableViewDataSource {
  func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
    return orders.count
  }
  
  func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
    let cell = tableView.dequeueReusableCell(withIdentifier: "cell", for: indexPath) as! DetailCell
    cell.productName.text = orders[indexPath.row].name
    cell.amount.text = "\(orders[indexPath.row].amount)"
    cell.price.text = "$\(orders[indexPath.row].price)"
    if let combo = orders[indexPath.row].combo {
      cell.combo.text = combo.joined(separator: "\n")
    }
    return cell
  }
}

class DetailCell: UITableViewCell {
  
  let productName: UILabel = {
    let label = UILabel()
    label.translatesAutoresizingMaskIntoConstraints = false
    label.textColor = .black
    label.numberOfLines = 2
    label.font = UIFont.systemFont(ofSize: 26, weight: .medium)
    return label
  }()
  
  let amount: UILabel = {
    let label = UILabel()
    label.translatesAutoresizingMaskIntoConstraints = false
    label.textColor = .black
    label.numberOfLines = 0
    label.textAlignment = .center
    label.font = UIFont.systemFont(ofSize: 26, weight: .medium)
    return label
  }()
  
  let price: UILabel = {
    let label = UILabel()
    label.translatesAutoresizingMaskIntoConstraints = false
    label.textColor = .black
    label.numberOfLines = 0
    label.textAlignment = .right
    label.font = UIFont.systemFont(ofSize: 26, weight: .medium)
    return label
  }()
  
  let combo: UILabel = {
    let label = UILabel()
    label.translatesAutoresizingMaskIntoConstraints = false
    label.textColor = .black
    label.numberOfLines = 0
    label.font = UIFont.systemFont(ofSize: 23, weight: .medium)
    return label
  }()
  
  override init(style: UITableViewCell.CellStyle, reuseIdentifier: String?) {
    super.init(style: style, reuseIdentifier: reuseIdentifier)
    selectionStyle = .none
    backgroundColor = .white
    contentView.addSubview(productName)
    productName.leadingAnchor.constraint(equalTo: leadingAnchor, constant: 10).isActive = true
    productName.heightAnchor.constraint(lessThanOrEqualToConstant: 60).isActive = true
    productName.topAnchor.constraint(equalTo: contentView.topAnchor, constant: 10).isActive = true
    productName.widthAnchor.constraint(equalTo: contentView.widthAnchor, multiplier: 0.5, constant: -10).isActive = true
    contentView.addSubview(combo)
    combo.topAnchor.constraint(equalTo: productName.bottomAnchor).isActive = true
    combo.leadingAnchor.constraint(equalTo: leadingAnchor, constant: 25).isActive = true
    combo.widthAnchor.constraint(equalTo: contentView.widthAnchor, multiplier: 0.5, constant: -25).isActive = true
    combo.bottomAnchor.constraint(equalTo: contentView.bottomAnchor).isActive = true
    contentView.addSubview(price)
    price.trailingAnchor.constraint(equalTo: trailingAnchor, constant: -15).isActive = true
    price.widthAnchor.constraint(equalTo: contentView.widthAnchor, multiplier: 0.3, constant: -15).isActive = true
    price.heightAnchor.constraint(equalToConstant: 30).isActive = true
    price.topAnchor.constraint(equalTo: contentView.topAnchor, constant: 10).isActive = true
    contentView.addSubview(amount)
    amount.trailingAnchor.constraint(equalTo: price.leadingAnchor).isActive = true
    amount.leadingAnchor.constraint(equalTo: productName.trailingAnchor).isActive = true
    amount.heightAnchor.constraint(equalToConstant: 30).isActive = true
    amount.topAnchor.constraint(equalTo: contentView.topAnchor, constant: 10).isActive = true
  }
  
  required init?(coder: NSCoder) {
    fatalError("init(coder:) has not been implemented")
  }
}
