//
//  ReceiptsController.swift
//  oin_pos_plugin
//
//  Created by 逸唐陳 on 2020/8/7.
//

import UIKit

class ReceiptsController: UIViewController {
  
  let tableView: UITableView = {
    let view = UITableView(frame: .zero, style: .plain)
    view.translatesAutoresizingMaskIntoConstraints = false
    view.separatorStyle = .none
    view.tableFooterView = UIView()
    view.backgroundColor = .white
    return view
  }()
  
  let imageView: UIImageView = {
    let view = UIImageView()
    view.translatesAutoresizingMaskIntoConstraints = false
    view.backgroundColor = .lightGray
    return view
  }()
  
  var screenshot: ((UIImage, UIImage) -> Void)?
  var receipts: Receipts!
  
  init(receipts: Receipts) {
    super.init(nibName: nil, bundle: nil)
    self.receipts = receipts
  }
  
  required init?(coder: NSCoder) {
    fatalError("init(coder:) has not been implemented")
  }
  
  override func viewDidLoad() {
    super.viewDidLoad()
    view.backgroundColor = .white
    navigationItem.title = "發票預覽"
    navigationItem.leftBarButtonItem = UIBarButtonItem(title: "返回", style: .plain, target: self, action: #selector(close))
    navigationItem.rightBarButtonItem = UIBarButtonItem(title: "列印", style: .plain, target: self, action: #selector(printHandler))
    view.addSubview(tableView)
    tableView.widthAnchor.constraint(equalToConstant: 420).isActive = true
//    tableView.leadingAnchor.constraint(equalTo: view.leadingAnchor).isActive = true
//    tableView.trailingAnchor.constraint(equalTo: view.trailingAnchor).isActive = true
    tableView.topAnchor.constraint(equalTo: view.topAnchor).isActive = true
    tableView.bottomAnchor.constraint(equalTo: view.bottomAnchor).isActive = true
    tableView.centerXAnchor.constraint(equalTo: view.centerXAnchor).isActive = true
    tableView.register(ReceiptsStoreCell.self, forCellReuseIdentifier: "store")
    tableView.register(ReceiptsDetailCell.self, forCellReuseIdentifier: "detail")
    tableView.register(ReceiptsDetailHeaderCell.self, forCellReuseIdentifier: "detailHeader")
    tableView.register(ReceiptsGiftCell.self, forCellReuseIdentifier: "gift")
    tableView.register(ReceiptsDiscountCell.self, forCellReuseIdentifier: "discount")
    tableView.register(ReceiptsTotalCell.self, forCellReuseIdentifier: "total")
    tableView.register(ReceiptsEventCell.self, forCellReuseIdentifier: "event")
    tableView.delegate = self
    tableView.dataSource = self
  }
  
  @objc func printHandler() {
    guard let info = tableView.asFullImage(at: 0) else {return}
    guard let detail = tableView.asFullImage(at: 1) else {return}
    screenshot?(info, detail)
//    view.addSubview(imageView)
//    imageView.centerXAnchor.constraint(equalTo: view.centerXAnchor).isActive = true
//    imageView.centerYAnchor.constraint(equalTo: view.centerYAnchor).isActive = true
//    imageView.heightAnchor.constraint(equalToConstant: 400).isActive = true
//    imageView.widthAnchor.constraint(lessThanOrEqualToConstant: 200).isActive = true
//    imageView.image = image
  }
  
  @objc func close() {
    navigationController?.dismiss(animated: true, completion: nil)
  }
}

let gifts = [Gift(name: "肉多多兌換券", amount: 1, price: "$100"), Gift(name: "100元折價券", amount: 1, price: "$0"), Gift(name: "200元折價券", amount: 1, price: "$0")]
let discounts = [Discount(name: "活動折價券", amount: 1, price: "-$1000")]

extension ReceiptsController: UITableViewDelegate {
  func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
    if indexPath.section == 0 {
      return UITableView.automaticDimension
    }else {
      switch indexPath.row {
      case 0:
        return UITableView.automaticDimension
      case 1:
        let tableHeight = 52 + receipts.nameHeight + receipts.comboHeight
        return tableHeight
      case 2:
        let tableHeight = 50 * gifts.count
        return CGFloat(tableHeight + 72)
      case 3:
        let tableHeight = 50 * discounts.count
        return CGFloat(tableHeight + 72)
      default:
        return UITableView.automaticDimension
      }
    }
  }
}
extension ReceiptsController: UITableViewDataSource {
  
  func numberOfSections(in tableView: UITableView) -> Int {
    return 2
  }
  
  func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
    if section == 0 {
      return 1
    }
    return 6
  }
  
  func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
    if indexPath.section == 0 {
      let cell = tableView.dequeueReusableCell(withIdentifier: "store", for: indexPath) as! ReceiptsStoreCell
      return cell
    }else {
      switch indexPath.row {
      case 0:
        // 交易明細 header
        let cell = tableView.dequeueReusableCell(withIdentifier: "detailHeader", for: indexPath) as! ReceiptsDetailHeaderCell
        return cell
      case 1:
        // 交易明細
        let cell = tableView.dequeueReusableCell(withIdentifier: "detail", for: indexPath) as! ReceiptsDetailCell
        cell.title.text = "消費明細 合計\(receipts.orders.count)項"
        cell.orders = receipts.orders
        return cell
      case 2:
        let cell = tableView.dequeueReusableCell(withIdentifier: "gift", for: indexPath) as! ReceiptsGiftCell
        cell.title.text = "贈品明細 合計\(gifts.count)項"
        cell.gifts = gifts
        return cell
      case 3:
        let cell = tableView.dequeueReusableCell(withIdentifier: "discount", for: indexPath) as! ReceiptsDiscountCell
        cell.title.text = "折扣明細 合計\(discounts.count)項"
        cell.discounts = discounts
        return cell
      case 4:
        let cell = tableView.dequeueReusableCell(withIdentifier: "total", for: indexPath) as! ReceiptsTotalCell
        return cell
      default:
        let cell = tableView.dequeueReusableCell(withIdentifier: "event", for: indexPath) as! ReceiptsEventCell
        return cell
      }
    }
  }
}

extension UITableView {
  
  func asFullImage(at section: Int) -> UIImage? {
    
    guard self.numberOfSections > 0, self.numberOfRows(inSection: section) > 0 else {
      return nil
    }
    
    self.scrollToRow(at: IndexPath(row: 0, section: section), at: .top, animated: false)
    
    var height: CGFloat = 0.0
//    for section in 0..<self.numberOfSections {
      var cellHeight: CGFloat = 0.0
      for row in 0..<self.numberOfRows(inSection: section) {
        let indexPath = IndexPath(row: row, section: section)
        guard let cell = self.cellForRow(at: indexPath) else { continue }
        cellHeight = cell.frame.size.height
        height += cellHeight
//        print(cellHeight)
        if row < self.numberOfRows(inSection: section) - 1 {
          self.scrollToRow(at: IndexPath(row: row+1, section: section), at: .bottom, animated: false)
        }
      }
//    print(height)
    
    UIGraphicsBeginImageContextWithOptions(CGSize(width: self.contentSize.width, height: height), false, 0)
    
//    for section in 0..<self.numberOfSections {
      var lastCellHeight: CGFloat = 0
      self.scrollToRow(at: IndexPath(row: 0, section: section), at: .top, animated: false)
      for row in 0..<self.numberOfRows(inSection: section) {
        let indexPath = IndexPath(row: row, section: section)
        guard let cell = self.cellForRow(at: indexPath) else { continue }
        let cellHeight = cell.frame.size.height
        let rect = CGRect(x: 0, y: lastCellHeight, width: cell.frame.width, height: cellHeight)
        lastCellHeight += cellHeight
//        print(lastCellHeight)
//        print(cellHeight)
        cell.contentView.drawHierarchy(in: rect, afterScreenUpdates: true)
        if row < self.numberOfRows(inSection: section) - 1 {
          self.scrollToRow(at: IndexPath(row: row+1, section: section), at: .bottom, animated: false)
        }
      }
//    }
    let image = UIGraphicsGetImageFromCurrentImageContext()
    UIGraphicsEndImageContext()
    
    return image!
  }
}
